/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Jakarta", "Avalon", "Excalibur" and "Apache Software Foundation"
    must not be used to endorse or promote products derived from this  software
    without  prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation. For more  information on the
 Apache Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.excalibur.mpool;

/**
 * This interface is to define how a Pool is used.  We have determined by
 * using the previous Pool implementations that the Pool marker interface
 * is considered harmful.  When generics are introduced in JDK 1.5, this
 * interface will be a prime candidate for those improvements.
 *
 * <p>
 *  It is important to realize that some objects are cheaper to simply allow
 *  the garbage collector to take care of them.  Therefore, only pool objects
 *  that are computationally expensive to create.  Prime candidates would be
 *  Components, JDBC Connection objects, Socket connections, etc.
 * </p>
 * <p>
 *  The interface is inspired by both the Mutex acquire/release and the
 *  structure of the ThreadLocal object.  In fact, it would be trivial
 *  to implement a "ThreadLocal" pool.
 * </p>
 *
 * @author <a href="mailto:bloritsch@apache.org">Berin Loritsch</a>
 * @version CVS $Revision: 1.9 $ $Date: 2003/03/22 12:46:28 $
 * @since 4.1.2
 */
public interface Pool
{
    /**
     * Acquire an instance of the pooled object.
     *
     * @return the pooled Object instance
     *
     * @throws Exception if the Pool is not able to return an object.
     */
    Object acquire() throws Exception;

    /**
     * Release the instance of the pooled object.
     *
     * @param pooledObject  The pooled object to release to the pool.
     */
    void release( Object pooledObject );

    /**
     * Create a new instance of the object being pooled.
     *
     * @return the pooled Object instance
     *
     * @throws Exception if the instance cannot be created
     */
    Object newInstance() throws Exception;
}
